#ifndef __Common_Avs25_Clip_H__
#define __Common_Avs25_Clip_H__

#include "../common/common.h"
#include "avisynth.h"

namespace Filtering { namespace Avisynth25 {

/* plane conversion */
template<typename T> Plane<T> ConvertTo(const PVideoFrame& frame, int nPlane);

template<> static inline Plane<Byte> ConvertTo<Byte>(const PVideoFrame &frame, int nPlane)
{
   return Plane<Byte>( frame->GetWritePtr( nPlane ), frame->GetPitch( nPlane ), frame->GetRowSize( nPlane ), frame->GetHeight( nPlane ) );
}
template<> static inline Plane<const Byte> ConvertTo<const Byte>(const PVideoFrame &frame, int nPlane)
{
   return Plane<const Byte>( frame->GetReadPtr( nPlane ), frame->GetPitch( nPlane ), frame->GetRowSize( nPlane ), frame->GetHeight( nPlane ) );
}

/* colorspace conversion */
Colorspace AVSColorspaceToColorspace(int pixel_type)
{
   switch ( pixel_type )
   {
   case VideoInfo::CS_I420: return COLORSPACE_I420;
   case VideoInfo::CS_YV12: return COLORSPACE_YV12;
   default: return COLORSPACE_NONE;
   }
}

static const int PlaneOrder[] = { PLANAR_Y, PLANAR_U, PLANAR_V };

/* clip class, adapted to avs25 */
class Clip : public Filtering::Clip {

   ::PClip pclip;
   IScriptEnvironment *env;

public:

   Clip(const ::PClip &pclip) : pclip(pclip), env(NULL)
   {
      VideoInfo vi = pclip->GetVideoInfo();
      nWidth = vi.width;
      nHeight = vi.height;
      nFrames = vi.num_frames;
      C = AVSColorspaceToColorspace( vi.pixel_type );
   }

   template<typename T> Frame<T> ConvertTo(const PVideoFrame& frame)
   {
      Plane<T> planes[3];
      for ( int i = 0; i < plane_counts[colorspace()]; i++ )
         planes[i] = Avisynth25::ConvertTo<T>( frame, PlaneOrder[i] ); 

      return Frame<T>( planes, C );
   }

   virtual Frame<Byte> get_frame(int n) { return ConvertTo<Byte>( pclip->GetFrame( n ,env ) ); }
   virtual Frame<const Byte> get_const_frame(int n) { return ConvertTo<const Byte>( pclip->GetFrame( n, env ) ); }
   void set_env(IScriptEnvironment *env) { this->env = env; }

};

} } // namespace Avisynth25, Common

#endif
