#ifndef __Mt_Symbol_H__
#define __Mt_Symbol_H__

#include "../utils/utils.h"

namespace Filtering { namespace Parser {

class Symbol {
public:
   typedef enum {
      NUMBER,
      OPERATOR,
      FUNCTION,
      TERNARY,
      VARIABLE_X,
      VARIABLE_Y,

      UNDEFINED

   } Type;

public:

   Type type;
   String value;
   int nParameter;
   double dValue;
   typedef double (*Process)(double x, double y, double z);
   Process process;

private:

private:
public:

   Symbol();
   Symbol(String value, Type type, int nParameter, Process process);
   Symbol(String value, double dValue, Type type, int nParameter, Process process);

   void setValue(double dValue);
   double getValue(double x, double y, double z) const;

   static Symbol Addition;
   static Symbol Multiplication;
   static Symbol Division;
   static Symbol Substraction;
   static Symbol Power;
   static Symbol Modulo;
   static Symbol Interrogation;
   static Symbol Equal;
   static Symbol NotEqual;
   static Symbol Inferior;
   static Symbol InferiorStrict;
   static Symbol Superior;
   static Symbol SuperiorStrict;
   static Symbol And;
   static Symbol Or;
   static Symbol AndNot;
   static Symbol Xor;
   static Symbol Pi;
   static Symbol X;
   static Symbol Y;
   static Symbol Cos;
   static Symbol Sin;
   static Symbol Tan;
   static Symbol Log;
   static Symbol Abs;
   static Symbol Exp;
   static Symbol Acos;
   static Symbol Atan;
   static Symbol Asin;
};

class Context : private std::list<double> {

   Symbol *pSymbols;
   int nSymbols;
   int nPos;

   double x, y;
   double rec_compute();
   double compute(double x, double y);

public:
   
   Context(const std::list<Symbol> &expression);

   ~Context();

   bool check();
   Byte compute_byte(double x, double y) { double f = compute(x, y); return f < 0.0f ? 0 : (f > 255.0f ? 255 : Byte(f + 0.5f)); }
};

} } // namespace Parser, Filtering

#endif
