#include "filter.h"
#include "../functions.h"

using namespace Filtering;

template<class T>
static void custom_c(Byte *pDst, int nDstPitch, const Byte *pSrc, int nSrcPitch, const Byte *pLut, const int *pCoordinates, int nCoordinates, int nWidth, int nHeight)
{
   T new_value( nCoordinates );
   for ( int j = 0; j < nHeight; j++ )
   {
      for ( int i = 0; i < nWidth; i++ )
      {
         new_value.reset();
         for ( int k = 0; k < nCoordinates; k+=2 )
         {
            if ( pCoordinates[k] + i >= 0 && pCoordinates[k] + i < nWidth &&
                 pCoordinates[k+1] + j >= 0 && pCoordinates[k+1] + j < nHeight )
               new_value.add( pLut[( pSrc[i] << 8 ) + pSrc[i + pCoordinates[k] + pCoordinates[k+1] * nSrcPitch]] );
         }
         pDst[i] = new_value.finalize();
      }
      pSrc += nSrcPitch;
      pDst += nDstPitch;
   }
}

namespace Filtering { namespace MaskTools { namespace Filters { namespace Lut { namespace Spatial {

Processor *averager_c = &custom_c<Averager<int> >;
Processor *minimizer_c = &custom_c<Minimizer>;
Processor *maximizer_c = &custom_c<Maximizer>;
Processor *deviater_c = &custom_c<Deviater<int> >;
Processor *ranger_c = &custom_c<Rangizer>;
Processor *medianizer_c = &custom_c<Medianizer>;
Processor *medianizer4_c = &custom_c<MedianizerBetter<4> >;
Processor *medianizer6_c = &custom_c<MedianizerBetter<6> >;
Processor *medianizer2_c = &custom_c<MedianizerBetter<2> >;

} } } } }

