%INCLUDE "common.asm"

SECTION .rodata:
ALIGN 16
   MT_WORD_128    : TIMES 8 DW 0x0080
   MT_WORD_256    : TIMES 8 DW 0x0100
   MT_BYTE_ZEROING: TIMES 8 DW 0x00FF

;=============================================================================
; Code
;=============================================================================

SECTION .text:

   mt_mangle Merge, merge4_mmx
   mt_mangle Merge, merge8_mmx
   mt_mangle Merge, merge8_isse
   mt_mangle Merge, merge8_3dnow
   mt_mangle Merge, merge_luma8_mmx
   mt_mangle Merge, merge_luma8_isse
   mt_mangle Merge, merge_luma8_3dnow

%MACRO MT_LUMA_LOAD_8   1
   movq                 mm2, [ebx + 2 * eax + 2 * %1]
   movq                 mm6, [ebx + 2 * eax + 2 * %1 + 8]
   movq                 mm1, [ebp + 2 * eax + 2 * %1]
   movq                 mm0, [ebp + 2 * eax + 2 * %1 + 8]
   pavgb                mm2, [ebx + 2 * eax + 2 * %1 + 1]
   pavgb                mm6, [ebx + 2 * eax + 2 * %1 + 1 + 8]
   pavgb                mm1, [ebp + 2 * eax + 2 * %1 + 1]
   pavgb                mm0, [ebp + 2 * eax + 2 * %1 + 1 + 8]
   pavgb                mm2, mm1
   pavgb                mm6, mm0
   pand                 mm2, [MT_BYTE_ZEROING]
   pand                 mm6, [MT_BYTE_ZEROING]
%ENDMACRO    

%MACRO MT_LOAD_8        1
   movd                 mm2, [ebx + eax + %1]
   movd                 mm6, [ebx + eax + 4 + %1]
%ENDMACRO

%MACRO MT_LUMA_END_LOAD 0
%ENDMACRO

%MACRO MT_END_LOAD      0
   punpcklbw            mm2, mm7
   punpcklbw            mm6, mm7
%ENDMACRO
   
%MACRO MT_SUB_MERGE_8   3
   movq                 mm3, [MT_WORD_256]
   
   %2                   %1
   
   movd                 mm1, [edi + eax + %1]
   movd                 mm5, [edi + eax + 4 + %1]
   movd                 mm0, [esi + eax + %1]
   movd                 mm4, [esi + eax + 4 + %1]

   %3
      
   punpcklbw            mm1, mm7
   punpcklbw            mm5, mm7
   psubw                mm3, mm2
   punpcklbw            mm0, mm7
   punpcklbw            mm4, mm7
   pmullw               mm1, mm3
   movq                 mm3, [MT_WORD_256]
   pmullw               mm4, mm6
   pmullw               mm0, mm2
   psubw                mm3, mm6
   paddw                mm4, [MT_WORD_128]
   pmullw               mm5, mm3
   paddw                mm0, [MT_WORD_128]
   paddw                mm4, mm5
   paddw                mm0, mm1
   psrlw                mm4, 8
   psrlw                mm0, 8

   packuswb             mm0, mm4
   
   movq                 [edi + eax + %1], mm0
%ENDMACRO

%MACRO MT_MERGE_8       1
   MT_SUB_MERGE_8       %1, MT_LOAD_8, MT_END_LOAD
%ENDMACRO

%MACRO MT_LUMA_MERGE_8  1
   MT_SUB_MERGE_8       %1, MT_LUMA_LOAD_8, MT_LUMA_END_LOAD
%ENDMACRO   

%MACRO MT_MERGE_ROLLED  0
   MT_MERGE_8           0
%ENDMACRO

%MACRO MT_LUMA_MERGE_ROLLED  0
   MT_LUMA_MERGE_8           0
%ENDMACRO

%MACRO MT_MERGE_UNROLLED 2
   %1                   edi + eax + 64
   %2                   esi + eax + 64
   %2                   ebx + eax + 64
   
   MT_MERGE_8           0
   MT_MERGE_8           8
   MT_MERGE_8           16
   MT_MERGE_8           24
   MT_MERGE_8           32
   MT_MERGE_8           40
   MT_MERGE_8           48
   MT_MERGE_8           56
%ENDMACRO

%MACRO MT_LUMA_MERGE_UNROLLED 2
   %1                   edi + eax + 64
   %2                   esi + eax + 64
   
   MT_LUMA_MERGE_8           0
   MT_LUMA_MERGE_8           8
   MT_LUMA_MERGE_8           16
   MT_LUMA_MERGE_8           24
   MT_LUMA_MERGE_8           32
   MT_LUMA_MERGE_8           40
   MT_LUMA_MERGE_8           48
   MT_LUMA_MERGE_8           56
%ENDMACRO

%MACRO MT_MERGE_ENDLINE 0
   add                  edi, [esp + .nDstPitch]
   add                  esi, [esp + .nSrc1Pitch]
   add                  ebx, [esp + .nSrc2Pitch]
%ENDMACRO

%MACRO MT_LUMA_MERGE_ENDLINE 0
   add                  edi, [esp + .nDstPitch]
   add                  esi, [esp + .nSrc1Pitch]
   add                  ebp, [esp + .nSrc2Pitch]
   mov                  ebx, ebp
   add                  ebp, [esp + .nSrc2Pitch]
%ENDMACRO

%MACRO MT_MERGE_COMMON  2
   MT_SET_STACK         0, esi, edi, ebx
   .pDst                PARAM(1)
   .nDstPitch           PARAM(2)
   .pSrc1               PARAM(3)
   .nSrc1Pitch          PARAM(4)
   .pSrc2               PARAM(5)
   .nSrc2Pitch          PARAM(6)
   .nWidth              PARAM(7)
   .nHeight             PARAM(8)

   pxor                 mm7, mm7
   
   mov                  edi, [esp + .pDst]
   mov                  esi, [esp + .pSrc1]
   mov                  ebx, [esp + .pSrc2]
   
   MT_UNROLL_INPLACE_WIDTH  {MT_MERGE_UNROLLED %1, %2}, MT_MERGE_ROLLED, MT_MERGE_ENDLINE
      
   MT_REL_STACK
%ENDMACRO

%MACRO MT_LUMA_MERGE_COMMON  2
   MT_SET_STACK         0, esi, edi, ebx
   .pDst                PARAM(1)
   .nDstPitch           PARAM(2)
   .pSrc1               PARAM(3)
   .nSrc1Pitch          PARAM(4)
   .pSrc2               PARAM(5)
   .nSrc2Pitch          PARAM(6)
   .nWidth              PARAM(7)
   .nHeight             PARAM(8)

   pxor                 mm7, mm7
   
   mov                  edi, [esp + .pDst]
   mov                  esi, [esp + .pSrc1]
   mov                  ebx, [esp + .pSrc2]
   mov                  ebp, ebx
   add                  ebp, [esp + .nSrc2Pitch]
   
   MT_UNROLL_INPLACE_WIDTH  {MT_LUMA_MERGE_UNROLLED %1, %2}, MT_LUMA_MERGE_ROLLED, MT_LUMA_MERGE_ENDLINE
      
   MT_REL_STACK
%ENDMACRO

merge8_mmx:
   MT_MERGE_COMMON      MT_NOPREFETCH, MT_NOPREFETCH

merge8_isse:
   MT_MERGE_COMMON      MT_PREFETCHT0, MT_PREFETCHT0

merge8_3dnow:
   MT_MERGE_COMMON      MT_PREFETCHW, MT_PREFETCHT0

merge_luma8_mmx:
   MT_LUMA_MERGE_COMMON MT_NOPREFETCH, MT_NOPREFETCH

merge_luma8_isse:
   MT_LUMA_MERGE_COMMON MT_PREFETCHT0, MT_PREFETCHT0

merge_luma8_3dnow:
   MT_LUMA_MERGE_COMMON MT_PREFETCHW, MT_PREFETCHT0

merge4_mmx:
   MT_SET_STACK         0, esi, edi, ebx, ebp
   .pDst                PARAM(1)
   .nDstPitch           PARAM(2)
   .pSrc1               PARAM(3)
   .nSrc1Pitch          PARAM(4)
   .pSrc2               PARAM(5)
   .nSrc2Pitch          PARAM(6)
   .nWidth              PARAM(7)
   .nHeight             PARAM(8)
   
   mov                  edi, [esp + .pDst]
   mov                  esi, [esp + .pSrc1]
   mov                  edx, [esp + .pSrc2]
   mov                  ecx, [esp + .nHeight]
   
   pxor                 mm7, mm7
   
.loopy

   mov                  eax, [esp + .nWidth]
   
.loopx

   sub                  eax, 4
   
   movd                 mm2, [edx + eax]
   movd                 mm0, [esi + eax]
   movd                 mm1, [edi + eax]
   movq                 mm3, [MT_WORD_256]
   movq                 mm4, [MT_WORD_128]
   
   punpcklbw            mm2, mm7
   punpcklbw            mm0, mm7
   punpcklbw            mm1, mm7
   
   psubw                mm3, mm2
   pmullw               mm0, mm2
   pmullw               mm1, mm3
   paddw                mm0, mm1
   paddw                mm0, mm4
   psrlw                mm0, 8
   
   packuswb             mm0, mm0
   
   movd                 [edi + eax], mm0
   
   test                 eax, eax
   jnz                  .loopx
   
   add                  edi, [esp + .nDstPitch]
   add                  esi, [esp + .nSrc1Pitch]
   add                  edx, [esp + .nSrc2Pitch]
   dec                  ecx
   jnz                  .loopy
   
   MT_REL_STACK