#ifndef __Mt_Softwire_H__
#define __Mt_Softwire_H__

#include "../common/common.h"

#include <softwire/softwire.hpp>

namespace MaskTools { namespace SoftWire {

class DynamicGenerator : public ::SoftWire::CodeGenerator
{
public:
   DynamicGenerator() {}
};

typedef ::SoftWire::Encoding *(DynamicGenerator::*LoadInstruction)(const ::SoftWire::OperandMMREG &, const ::SoftWire::OperandMEM64 &);

void loadHeightPixels(DynamicGenerator &x86, LoadInstruction load, const ::SoftWire::OperandMMREG &dest, 
                      const ::SoftWire::OperandREF &address, const ::SoftWire::OperandMMREG &junk1,
                      const ::SoftWire::OperandMMREG &junk2, int nX, int nWidth);

class Lines3 : public DynamicGenerator
{
   ::SoftWire::OperandREG32 oPitch;
   ::SoftWire::OperandREG32 oPtr;
   int nWidth, nHeight;
   int nX, nY;
   int nJumpDisp;
   DynamicGenerator &x86;
public:
   Lines3(int nWidth, int nHeight, DynamicGenerator &x86, const ::SoftWire::OperandREG32 &oPitch,
          const ::SoftWire::OperandREG32 &oPtr1, const ::SoftWire::OperandREG32 &oPtr2) : nWidth(nWidth), nHeight(nHeight), nX(0), nY(0), x86(x86), oPitch(oPitch), oPtr(oPtr1) {}
   int stack_size() const;
   void prologue(int nStackDisp, int nPtrDisp, int nPitchDisp);
   void epilogue();
   void nextLine();
   void nextPixels(int nStride);
   void loadHeightPixels(int nX, int nY, const ::SoftWire::OperandREG32 &oHorOffset, LoadInstruction load,
                         const ::SoftWire::OperandMMREG &dest, const ::SoftWire::OperandMMREG &junk1, const ::SoftWire::OperandMMREG &junk2) const;
};

class Lines1 : public DynamicGenerator
{
   ::SoftWire::OperandREG32 oPitch;
   ::SoftWire::OperandREG32 oPtr;
   int nWidth, nHeight;
   int nX, nY;
   DynamicGenerator &x86;
public:
   Lines1(int nWidth, int nHeight, DynamicGenerator &x86, const ::SoftWire::OperandREG32 &oPitch,
          const ::SoftWire::OperandREG32 &oPtr1, const ::SoftWire::OperandREG32 &oPtr2) : nWidth(nWidth), nHeight(nHeight), nX(0), nY(0), x86(x86), oPitch(oPitch), oPtr(oPtr1) {}
   int stack_size() const;
   void prologue(int nStackDisp, int nPtrDisp, int nPitchDisp);
   void epilogue();
   void nextLine();
   void nextPixels(int nStride);
   void loadHeightPixels(int nX, int nY, const ::SoftWire::OperandREG32 &oHorOffset, LoadInstruction load,
                         const ::SoftWire::OperandMMREG &dest, const ::SoftWire::OperandMMREG &junk1, const ::SoftWire::OperandMMREG &junk2) const;
};

} }

#endif

